@@ -7,6 +7,7 @@ import argparse
 import enum
 import os
 import yaml
+import asyncio
 
 import bluesky_queueserver
 from .comms import (
@@ -17,6 +18,8 @@ from .comms import (
     default_zmq_control_address,
 )
 
+from .plan_queue_ops import PlanQueueOperations
+
 import logging
 
 logger = logging.getLogger(__name__)
@@ -147,10 +150,22 @@ Example of JSON specification of a function ("args" and "kwargs" are optional):
 qserver script upload <path-to-file>              # Upload a script to RE Worker environment
 qserver script upload <path-to-file> background   # ... in the background
 qserver script upload <path-to-file> update-re    # ... allow 'RE' and 'db' to be updated
+qserver script upload <path-to-file> keep-lists   # ... leave lists of allowed and existing plans and devices
+                                                  #   unchanged (saves processing time)
 
 qserver task result <task-uid>  # Load status or result of a task with the given UID
 qserver task status <task-uid>  # Check status of a task with the given UID
 
+qserver lock environment  -k 90g94                   # Lock the environment
+qserver lock environment "Locked for 1 hr" -k 90g94  # Add a text note
+qserver lock queue -k 90g94                          # Lock the queue
+qserver lock all -k 90g94                            # Lock environment and the queue
+
+qserver lock info                        # Load lock status
+qserver lock info -k 90g94               # Load lock status and validate the key
+
+qserver unlock -k 90g94                  # Unlock RE Manager
+
 qserver manager stop           # Safely exit RE Manager application
 qserver manager stop safe on   # Safely exit RE Manager application
 qserver manager stop safe off  # Force RE Manager application to stop
@@ -512,7 +527,7 @@ def msg_function_execute(params, *, cmd_opt):
     return method, prms
 
 
-def msg_queue_item(params):
+def msg_queue_item(params, *, lock_key):
     """
     Generate outgoing message for `queue item` command. The supported options are ``get``,
     ``move`` and ``remove``.
@@ -593,6 +608,10 @@ def msg_queue_item(params):
     method = f"{command}_{params[0]}_{params[1]}"
     prms = addr_param
 
+    if p_item_type in ("remove", "move"):
+        if lock_key:
+            prms["lock_key"] = lock_key
+
     return method, prms
 
 
@@ -733,22 +752,25 @@ def msg_re_pause(params):
     return method, prms
 
 
-def create_msg(params):
+def create_msg(params, *, lock_key):
     """
     Create outgoing message based on command-line arguments.
 
     Parameters
     ----------
-    params : list
+    params: list
         List of parameters (positional arguments) supplied via command line.
 
+    lock_key: str or None
+        Lock key
+
     Returns
     -------
-    method : str
+    method: str
         The name of the method.
-    prms : dict
+    prms: dict
         The dictionary of method parameters.
-    monitoring_mode : bool
+    monitoring_mode: bool
         Indicates if monitoring mode is enabled.
 
     Raises
@@ -795,6 +817,8 @@ def create_msg(params):
         supported_params = ("open", "close", "destroy")
         if params[0] in supported_params:
             method = f"{command}_{params[0]}"
+            if lock_key:
+                prms["lock_key"] = lock_key
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
 
@@ -830,6 +854,8 @@ def create_msg(params):
             if len(params) == 2:
                 if params[1] == "lists":
                     prms["restore_plans_devices"] = True
+                    if lock_key:
+                        prms["lock_key"] = lock_key
                 else:
                     CommandParameterError(f"Request '{command} {params[0]} {params[1]}' is not supported")
 
@@ -853,6 +879,8 @@ def create_msg(params):
                     f"Request '{command}': failed to read the user group permissions from file '{file_name}': {ex}"
                 )
             prms["user_group_permissions"] = permissions_dict
+            if lock_key:
+                prms["lock_key"] = lock_key
 
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
@@ -874,7 +902,8 @@ def create_msg(params):
 
             run_in_background = False
             update_re = False
-            allowed_values = ("background", "update-re")
+            update_lists = True
+            allowed_values = ("background", "update-re", "keep-lists")
 
             for p in params[2:]:
                 if p not in allowed_values:
@@ -885,9 +914,17 @@ def create_msg(params):
                     run_in_background = True
                 elif p == "update-re":
                     update_re = True
+                elif p == "keep-lists":
+                    update_lists = False
 
             method = f"{command}_{params[0]}"
-            prms = {"script": script, "run_in_background": run_in_background, "update_re": update_re}
+            prms = {
+                "script": script,
+                "run_in_background": run_in_background,
+                "update_re": update_re,
+                "update_lists": update_lists,
+                "lock_key": lock_key,
+            }
 
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
@@ -913,6 +950,8 @@ def create_msg(params):
             raise CommandParameterError(f"Request '{command}' must include at least one parameter")
         if params[0] == "execute":
             method, prms = msg_function_execute(params, cmd_opt=params[0])
+            if lock_key:
+                prms["lock_key"] = lock_key
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
 
@@ -924,20 +963,29 @@ def create_msg(params):
         if params[0] in supported_params:
             if params[0] in ("add", "update", "replace", "execute"):
                 method, prms = msg_queue_add_update(params, cmd_opt=params[0])
+                if lock_key:
+                    prms["lock_key"] = lock_key
 
             elif params[0] in ("get", "clear", "start"):
                 if len(params) != 1:
                     raise CommandParameterError(f"Request '{command} {params[0]}' must include only one parameter")
                 method, prms = f"{command}_{params[0]}", {}
+                if params[0] in ("clear", "start"):
+                    if lock_key:
+                        prms["lock_key"] = lock_key
 
             elif params[0] == "stop":
                 method, prms = msg_queue_stop(params)
+                if lock_key:
+                    prms["lock_key"] = lock_key
 
             elif params[0] == "item":
-                method, prms = msg_queue_item(params)
+                method, prms = msg_queue_item(params, lock_key=lock_key)
 
             elif params[0] == "mode":
                 method, prms = msg_queue_mode(params)
+                if lock_key:
+                    prms["lock_key"] = lock_key
 
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
@@ -961,6 +1009,10 @@ def create_msg(params):
                     )
             else:
                 method, prms = f"{command}_{params[0]}", {}
+
+            if lock_key:
+                prms["lock_key"] = lock_key
+
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
 
@@ -970,6 +1022,9 @@ def create_msg(params):
         supported_params = ("get", "clear")
         if params[0] in supported_params:
             method, prms = f"{command}_{params[0]}", {}
+            if params[0] == "clear":
+                if lock_key:
+                    prms["lock_key"] = lock_key
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
 
@@ -998,6 +1053,48 @@ def create_msg(params):
         else:
             raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
 
+    elif command == "lock":
+        if len(params) == 0:
+            raise CommandParameterError("Command {command!r} requires at least one parameter.")
+        elif params[0] == "info":
+            if len(params) != 1:
+                raise CommandParameterError(
+                    f"Too many parameters (only one expected): {format_list_as_command(params)}"
+                )
+            method = "lock_info"
+            if lock_key:
+                prms["lock_key"] = lock_key
+        elif params[0] in ("environment", "queue", "all"):
+            if len(params) > 2:
+                raise CommandParameterError(
+                    f"Too many parameters (1 or 2 are expected): {format_list_as_command(params)}"
+                )
+            if (len(params) == 2) and (params[1] in ("environment", "queue", "all")):
+                raise CommandParameterError(
+                    f"Unsupported combination of parameters:  {format_list_as_command(params)}"
+                )
+            method = "lock"
+            if params[0] in ("environment", "all"):
+                prms["environment"] = True
+            if params[0] in ("queue", "all"):
+                prms["queue"] = True
+            if len(params) == 2:
+                prms["note"] = params[1]
+            if lock_key:
+                prms["lock_key"] = lock_key
+            prms["user"] = default_user
+        else:
+            raise CommandParameterError(f"Request '{command} {params[0]}' is not supported")
+
+    elif command == "unlock":
+        if len(params) != 0:
+            raise CommandParameterError(
+                f"The command accepts no extra parameters: {format_list_as_command(params)}"
+            )
+        method = "unlock"
+        if lock_key:
+            prms["lock_key"] = lock_key
+
     else:
         raise CommandParameterError(f"Unrecognized command '{command}'")
 
@@ -1082,6 +1179,16 @@ def qserver():
         help="The parameter is deprecated and will be removed. Use --zmq-control-addr instead.",
     )
 
+    parser.add_argument(
+        "--lock-key",
+        "-k",
+        dest="lock_key",
+        action="store",
+        default=None,
+        help="Lock key. The key is an arbitrary string is used to lock and unlock RE Manager "
+        "('lock' and 'unlock' API) and control the manager when the environment or the queue is locked.",
+    )
+
     args = parser.parse_args()
     print(f"Arguments: {args.command}")
 
@@ -1097,6 +1204,13 @@ def qserver():
         # If the address is not specified, then use the default address
         address = address or default_zmq_control_address
 
+        lock_key = args.lock_key
+        if lock_key is not None:
+            if not isinstance(lock_key, str) or not lock_key:
+                raise CommandParameterError(
+                    f"Lock key must be a non-empty string: submitted lock key is {lock_key!r}"
+                )
+
         # Read public key from the environment variable, then check if the CLI parameter exists
         zmq_public_key = os.environ.get("QSERVER_ZMQ_PUBLIC_KEY", None)
         zmq_public_key = zmq_public_key if zmq_public_key else None  # Case of key==""
@@ -1106,7 +1220,7 @@ def qserver():
             except Exception as ex:
                 raise CommandParameterError(f"ZMQ public key is improperly formatted: {ex}")
 
-        method, params, monitoring_mode = create_msg(args.command)
+        method, params, monitoring_mode = create_msg(args.command, lock_key=lock_key)
         if monitoring_mode:
             print("Running QServer monitor. Press Ctrl-C to exit ...")
 
@@ -1196,3 +1310,56 @@ def qserver_zmq_keys():
         return QServerExitCodes.EXCEPTION_OCCURRED.value
 
     return QServerExitCodes.SUCCESS.value
+
+
+def qserver_clear_lock():
+
+    logging.basicConfig(level=logging.WARNING)
+    logging.getLogger("bluesky_queueserver").setLevel("INFO")
+
+    def formatter(prog):
+        # Set maximum width such that printed help mostly fits in the RTD theme code block (documentation).
+        return argparse.RawDescriptionHelpFormatter(prog, max_help_position=20, width=90)
+
+    parser = argparse.ArgumentParser(
+        description="Bluesky-QServer: Clear RE Manager lock.\n"
+        f"bluesky-queueserver version {qserver_version}.\n\n"
+        "Recover locked RE Manager if the lock key is lost. The utility requires access to Redis\n"
+        "used by RE Manager. Provide the address of Redis service using '--redis-addr' parameter.\n"
+        "Restart the RE Manager service after clearing the lock.\n",
+        formatter_class=formatter,
+    )
+
+    parser.add_argument(
+        "--redis-addr",
+        dest="redis_addr",
+        type=str,
+        default="localhost",
+        help="The address of Redis server, e.g. 'localhost', '127.0.0.1', 'localhost:6379' "
+        "(default: %(default)s). ",
+    )
+
+    args = parser.parse_args()
+
+    exit_code = 0
+
+    async def remove_lock():
+        nonlocal exit_code
+        try:
+            pq = PlanQueueOperations(redis_host=args.redis_addr)
+            await pq.start()
+            lock_info = await pq.lock_info_retrieve()
+            if lock_info is not None:
+                print(f"Detected lock info:\n{pprint.pformat(lock_info)}")
+                print("Clearing the lock ...")
+                await pq.lock_info_clear()
+                print("RE Manager lock was cleared. Restart RE Manager service to unlock the manager!")
+            else:
+                print("No lock was detected.")
+        except Exception as ex:
+            exit_code = 1
+            logger.error(f"Failed to clear RE Manager lock: {ex}")
+
+    asyncio.run(remove_lock())
+
+    return exit_code
